void buttonUpdate() {

  currentMillis = millis();
  static unsigned int button_StateOnCount = 0; // to handle new pressed state, and still pressed state
  static unsigned int button_OnTimeLast = 0; // to handle new released state, and remember last on-time (button pressed)

  // poll button state, return continuous on-time [ms] if pressed (debounced)
  unsigned int button_OnTime = button.process(currentMillis);

  if (button_OnTime) {
    // save current on-time (button pressed), to be used when released
    button_OnTimeLast = button_OnTime;
    // check for state change
    unsigned int count = button.getStateOnCount();
    if (button_StateOnCount > count) {
      button_StateOnCount = count;
    }
    // handle pressed state
    if (button_OnTimeLast > sleepDelay) {
      digitalWrite(ledPin, HIGH);
    }
  }

  else {
    if (button_OnTimeLast) {
      // handle released state
      if (button_OnTimeLast > sleepDelay) {
        sleepNow();                                    //sleep mode function is called here
      }
    }
    button_OnTimeLast = 0; // reset, do not handle this released state again
  }
}

void sleepNow() {

  set_sleep_mode(SLEEP_MODE_PWR_DOWN);   // sleep mode is set here

  sleep_enable();          // enables the sleep bit in the mcucr register
  // so sleep is possible. just a safety pin

  attachInterrupt(0, wakeUpNow, LOW); // use interrupt 0 (pin 2) and run function
  // wakeUpNow when pin 2 gets LOW
  
  crossFade(black);                  //fades the LEDs off
  
  anodeOff();                      //powering all the output pins down
  digitalWrite(redled1, LOW);
  digitalWrite(greenled1, LOW);
  digitalWrite(blueled1, LOW);
  digitalWrite(redled2, LOW);
  digitalWrite(greenled2, LOW);
  digitalWrite(blueled2, LOW);

  digitalWrite(ledPin, LOW);
  sleep_mode();            // here the device is actually put to sleep!!

  // THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP

  anodeOn();             //switching the anode pins on

  colour = 1;
     
  sleep_disable();         // first thing after waking from sleep:
  // disable sleep...
  detachInterrupt(0);    // disables interrupt 0 on pin 2 so the
  // wakeUpNow code will not be executed
  // during normal running time.

}

void wakeUpNow() {       // here the interrupt is handled after wakeup

  // execute code here after wake-up before returning to the loop() function
  // timers and code using timers (serial.print and more...) will not work here.
  // we don't really need to execute any special functions here, since we
  // just want the thing to wake up
}

void anodeOn() {
  digitalWrite(ledanode1, LOW);
  digitalWrite(ledanode2, LOW);
  digitalWrite(ledanode3, LOW);
  digitalWrite(ledanode4, LOW);
  digitalWrite(ledanode5, LOW);
  digitalWrite(ledanode6, LOW);
  digitalWrite(ledanode7, LOW);
}

void anodeOff() {
  digitalWrite(ledanode1, HIGH);
  digitalWrite(ledanode2, HIGH);
  digitalWrite(ledanode3, HIGH);
  digitalWrite(ledanode4, HIGH);
  digitalWrite(ledanode5, HIGH);
  digitalWrite(ledanode6, HIGH);
  digitalWrite(ledanode7, HIGH);
}

/* BELOW THIS LINE IS THE MATH -- YOU SHOULDN'T NEED TO CHANGE THIS FOR THE BASICS

  The program works like this:
  Imagine a crossfade that moves the red LED from 0-10,
    the green from 0-5, and the blue from 10 to 7, in
    ten steps.
    We'd want to count the 10 steps and increase or
    decrease color values in evenly stepped increments.
    Imagine a + indicates raising a value by 1, and a -
    equals lowering it. Our 10 step fade would look like:

    1 2 3 4 5 6 7 8 9 10
  R + + + + + + + + + +
  G   +   +   +   +   +
  B     -     -     -

  The red rises from 0 to 10 in ten steps, the green from
  0-5 in 5 steps, and the blue falls from 10 to 7 in three steps.

  In the real program, the color percentages are converted to
  0-255 values, and there are 1020 steps (255*4).

  To figure out how big a step there should be between one up- or
  down-tick of one of the LED values, we call calculateStep(),
  which calculates the absolute gap between the start and end values,
  and then divides that gap by 1020 to determine the size of the step
  between adjustments in the value.
*/

int calculateStep(int prevValue, int endValue) {
  int step = endValue - prevValue; // What's the overall gap?
  if (step) {                      // If its non-zero,
    step = 1020 / step;            //   divide by 1020
  }
  return step;
}

/* The next function is calculateVal. When the loop value, i,
   reaches the step size appropriate for one of the
   colors, it increases or decreases the value of that color by 1.
   (R, G, and B are each calculated separately.)
*/

int calculateVal(int step, int val, int i) {

  if ((step) && i % step == 0) { // If step is non-zero and its time to change a value,
    if (step > 0) {              //   increment the value if step is positive...
      val += 1;
    }
    else if (step < 0) {         //   ...or decrement it if step is negative
      val -= 1;
    }
  }
  // Defensive driving: make sure val stays in the range 0-255
  if (val > 255) {
    val = 255;
  }
  else if (val < 0) {
    val = 0;
  }
  return val;
}

/* crossFade() converts the percentage colors to a
   0-255 range, then loops 1020 times, checking to see if
   the value needs to be updated each time, then writing
   the color values to the correct pins.
*/

void crossFade(int color[3]) {
  // Convert to 0-255
  int R = (color[0] * 255) / 100;
  int G = (color[1] * 255) / 100;
  int B = (color[2] * 255) / 100;

  int stepR = calculateStep(prevR, R);
  int stepG = calculateStep(prevG, G);
  int stepB = calculateStep(prevB, B);

  for (int i = 0; i <= 1020; i++) {
    redVal = calculateVal(stepR, redVal, i);
    grnVal = calculateVal(stepG, grnVal, i);
    bluVal = calculateVal(stepB, bluVal, i);

    analogWrite(redled1, redVal);   // Write current values to LED pins
    analogWrite(greenled1, grnVal);
    analogWrite(blueled1, bluVal);
    analogWrite(redled2, redVal);
    analogWrite(greenled2, grnVal);
    analogWrite(blueled2, bluVal);
    delayMicroseconds(wait); // Pause for 'wait' milliseconds before resuming the loop
  }
  // Update current values for next loop
  prevR = redVal;
  prevG = grnVal;
  prevB = bluVal;
}
